package cd.casic.ci.common.pipeline.type.agent;

import cd.casic.ci.common.pipeline.type.docker.ImageType;
import cd.casic.ci.common.pipeline.utils.EnvUtils;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import lombok.Getter;
import lombok.Setter;

import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.type.agent
 * @Project：ops-pro
 * @name：ThirdPartyAgentDockerInfo
 * @Date：2025/03/27 10:47
 * @Filename：ThirdPartyAgentDockerInfo
 */
@Getter
@Setter
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown = true)
public class ThirdPartyAgentDockerInfo {
    private String image;
    private Credential credential;
    private DockerOptions options;
    private String imagePullPolicy;
    private ImageType imageType = ImageType.THIRD;
    private ThirdPartyAgentDockerInfoStoreImage storeImage;

    public void replaceField(Map<String, String> variables) {
        image = EnvUtils.parseEnv(image, variables);
        if (credential != null) {
            if (!credential.getUser().isBlank()) {
                credential.setUser(EnvUtils.parseEnv(credential.getUser(), variables));
            }
            if (!credential.getPassword().isBlank()) {
                credential.setPassword(EnvUtils.parseEnv(credential.getPassword(), variables));
            }
            if (!credential.getCredentialId().isBlank()) {
                credential.setCredentialId(EnvUtils.parseEnv(credential.getCredentialId(), variables));
            }
        }
        if (options != null) {
            options.setVolumes(options.getVolumes().stream().map(v -> EnvUtils.parseEnv(v, variables)).toList());
            options.setMounts(options.getMounts().stream().map(m -> EnvUtils.parseEnv(m, variables)).toList());
            options.setGpus(EnvUtils.parseEnv(options.getGpus(), variables));
            options.setPrivileged(Boolean.parseBoolean(EnvUtils.parseEnv(String.valueOf(options.getPrivileged()), variables)));
        }
        if (imagePullPolicy != null) {
            imagePullPolicy = EnvUtils.parseEnv(imagePullPolicy, variables);
        }
    }
}

@Getter
@Setter
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown = true)
class Credential {
    private String user;
    private String password;
    private String credentialId;
    private String acrossTemplateId;
    private String jobId;
    private String credentialProjectId;
}

@Getter
@Setter
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown = true)
class DockerOptions {
    private List<String> volumes;
    private List<String> mounts;
    private String gpus;
    private Boolean privileged;
}

@Getter
@Setter
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown = true)
class ThirdPartyAgentDockerInfoStoreImage {
    private String imageName;
    private String imageCode;
    private String imageVersion;
}

@Getter
@Setter
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown = true)
class ThirdPartyAgentDockerInfoDispatch {
    private String agentId;
    private String secretKey;
    private String image;
    private Credential credential;
    private DockerOptions options;
    private String imagePullPolicy;

    public ThirdPartyAgentDockerInfoDispatch(String agentId, String secretKey, ThirdPartyAgentDockerInfo info) {
        this.agentId = agentId;
        this.secretKey = secretKey;
        this.image = info.getImage();
        this.credential = info.getCredential();
        this.options = info.getOptions();
        this.imagePullPolicy = info.getImagePullPolicy();
    }
}
