/*
 * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available.
 *
 * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license.
 *
 * A copy of the MIT License is included in this file.
 *
 * ...
 */

package cd.casic.devops.common.worker.env;

import cd.casic.ci.common.enums.OSType;
import lombok.SneakyThrows;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileInputStream;
import java.util.Locale;
import java.util.Properties;

public class AgentEnv {
    private static final Logger logger = LoggerFactory.getLogger(AgentEnv.class);

    public static final String PROJECT_ID = "devops.project.id";
    public static final String DOCKER_PROJECT_ID = "devops_project_id";
    public static final String AGENT_ID = "devops.agent.id";
    public static final String DOCKER_AGENT_ID = "devops_agent_id";
    public static final String AGENT_SECRET_KEY = "devops.agent.secret.key";
    public static final String DOCKER_AGENT_SECRET_KEY = "devops_agent_secret_key";
    public static final String AGENT_GATEWAY = "landun.gateway";
    public static final String DOCKER_GATEWAY = "devops_gateway";
    public static final String AGENT_FILE_GATEWAY = "DEVOPS_FILE_GATEWAY";
    public static final String AGENT_ENV = "landun.env";
    public static final String AGENT_LOG_SAVE_MODE = "devops_log_save_mode";
    public static final String AGENT_PROPERTIES_FILE_NAME = ".agent.properties";
    public static final String BK_TAG = "devops_bk_tag";
    public static final String PUBLIC_HOST_MAX_FILE_CACHE_SIZE = "devops.public.host.maxFileCacheSize";
    public static final String THIRD_HOST_MAX_FILE_CACHE_SIZE = "devops.public.third.maxFileCacheSize";

    private static String projectId;
    private static String agentId;
    private static String secretKey;
    private static String gateway;
    private static String fileGateway;
    private static OSType os;
//    private static Env env;
//    private static LogStorageMode logStorageMode;
    private static String bkTag;
    private static Properties property;

    private static final File propertyFile = new File(System.getProperty("user.home"), AGENT_PROPERTIES_FILE_NAME);

    public static String getProjectId() {
        if (projectId == null) {
            synchronized (AgentEnv.class) {
                if (projectId == null) {
                    projectId = getProperty(PROJECT_ID);
                    logger.info("Get the project ID({})", projectId);
                }
            }
        }
        return projectId;
    }

    public static String getAgentId() {
        if (agentId == null) {
            synchronized (AgentEnv.class) {
                if (agentId == null) {
                    agentId = getProperty(AGENT_ID);
                    logger.info("Get the agent ID({})", agentId);
                }
            }
        }
        return agentId;
    }

    public static String getSecretKey() {
        if (secretKey == null) {
            synchronized (AgentEnv.class) {
                if (secretKey == null) {
                    secretKey = getProperty(AGENT_SECRET_KEY);
                    logger.info("Get the secret key({})", secretKey);
                }
            }
        }
        return secretKey;
    }

    public static String getGateway() {
        if (gateway == null) {
            synchronized (AgentEnv.class) {
                if (gateway == null) {
                    gateway = getProperty(AGENT_GATEWAY);
                    logger.info("Get the gateway({})", gateway);
                }
            }
        }
        return gateway;
    }

    public static String getFileGateway() {
        if (fileGateway == null) {
            synchronized (AgentEnv.class) {
                if (fileGateway == null) {
                    fileGateway = getProperty(AGENT_FILE_GATEWAY);
                    logger.info("Get the file gateway({})", fileGateway);
                }
            }
        }
        return fileGateway;
    }

    public static OSType getOs() {
        if (os == null) {
            synchronized (AgentEnv.class) {
                if (os == null) {
                    os = OSType.valueOf(System.getProperty("os.name").toUpperCase(Locale.ROOT));
                    logger.info("Get the OS type({})", os);
                }
            }
        }
        return os;
    }

  /*  public static LogStorageMode getLogMode() {
        if (logStorageMode == null) {
            synchronized (AgentEnv.class) {
                if (logStorageMode == null) {
                    logStorageMode = LogStorageMode.valueOf(getProperty(AGENT_LOG_SAVE_MODE));
                    logger.info("Get the log mode({})", logStorageMode);
                }
            }
        }
        return logStorageMode;
    }*/

    public static String getBkTag() {
        if (bkTag == null) {
            synchronized (AgentEnv.class) {
                if (bkTag == null) {
                    bkTag = getProperty(BK_TAG);
                    logger.info("Get the bkTag({})", bkTag);
                }
            }
        }
        return bkTag;
    }

    @SneakyThrows
    public static String getProperty(String prop) {
        if (property == null) {
            if (!propertyFile.exists()) {
                throw new Exception("The property file(" + propertyFile.getAbsolutePath() + ") does not exist");
//                throw new ParamBlankException("The property file(" + propertyFile.getAbsolutePath() + ") does not exist");
            }
            property = new Properties();
            try (FileInputStream fis = new FileInputStream(propertyFile)) {
                property.load(fis);
            } catch (Exception e) {
                logger.error("Failed to load properties from file", e);
            }
        }
        return property.getProperty(prop);
    }

    public static String getEnvProp(String prop) {
        String value = System.getenv(prop);
        if (value == null || value.isEmpty()) {
            value = System.getProperty(prop);
        }
        return value;
    }

    public static boolean isDockerEnv() {
        return BuildEnv.getBuildType() == BuildType.DOCKER;
    }

    public static String getLocaleLanguage() {
        return null;
//        return System.getProperty(LOCALE_LANGUAGE, System.getenv(LOCALE_LANGUAGE) != null ? System.getenv(LOCALE_LANGUAGE) : DEFAULT_LOCALE_LANGUAGE);
    }
} 