/*
 * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available.
 *
 * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license.
 *
 * A copy of the MIT License is included in this file.
 *
 * ...
 */

package cd.casic.devops.common.worker.env;

import cd.casic.framework.commons.exception.ServerException;
import cd.casic.framework.commons.exception.ServiceException;
import lombok.SneakyThrows;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DockerEnv {
    private static final Logger logger = LoggerFactory.getLogger(DockerEnv.class);

    private static final String PROJECT_ID = "devops_project_id";
    private static final String BUILD_ID = "devops_build_id";
    private static final String AGENT_ID = "devops_agent_id";
    private static final String AGENT_SECRET_KEY = "devops_agent_secret_key";
    private static final String AGENT_GATEWAY = "devops_gateway";
    private static final String JOB_POOL = "JOB_POOL";
    private static final String DOCKER_HOST_IP = "docker_host_ip";
    private static final String DOCKER_HOST_PORT = "docker_host_port";
    private static final String HOSTNAME = "HOSTNAME";

    private static String projectId;
    private static String buildId;
    private static String agentId;
    private static String secretKey;
    private static String gateway;
    private static String jobPool;
    private static String dockerHostIp;
    private static String dockerHostPort;
    private static String hostname;

    public static void setProjectId(String projectId) {
        System.setProperty(PROJECT_ID, projectId);
    }

    @SneakyThrows
    public static String getProjectId() {
        if (projectId == null) {
            synchronized (DockerEnv.class) {
                if (projectId == null) {
                    projectId = getProperty(PROJECT_ID);
                    if (projectId == null || projectId.isEmpty()) {
                        throw new Exception(PROJECT_ID + "Empty project Id");
//                        throw new PropertyNotExistException(PROJECT_ID, "Empty project Id");
                    }
                    logger.info("Get the project ID({})", projectId);
                }
            }
        }
        return projectId;
    }

    public static void setBuildId(String buildId) {
        System.setProperty(BUILD_ID, buildId);
    }

    @SneakyThrows
    public static String getBuildId() {
        if (buildId == null) {
            synchronized (DockerEnv.class) {
                if (buildId == null) {
                    buildId = getProperty(BUILD_ID);
                    if (buildId == null || buildId.isEmpty()) {
                        throw new Exception(BUILD_ID + "Empty build Id");
//                        throw new PropertyNotExistException(BUILD_ID, "Empty build Id");
                    }
                    logger.info("Get the build ID({})", buildId);
                }
            }
        }
        return buildId;
    }

    public static String getAgentId() {
        if (agentId == null) {
            synchronized (DockerEnv.class) {
                if (agentId == null) {
                    agentId = getProperty(AGENT_ID);
                    if (agentId == null || agentId.isEmpty()) {
                        throw new ServerException(-1000, AGENT_ID + "Empty agent Id");
                    }
                    logger.info("Get the agent ID({})", agentId);
                }
            }
        }
        return agentId;
    }

    public static void setAgentSecretKey(String secretKey) {
        System.setProperty(AGENT_SECRET_KEY, secretKey);
    }

    public static String getAgentSecretKey() {
        if (secretKey == null) {
            synchronized (DockerEnv.class) {
                if (secretKey == null) {
                    secretKey = getProperty(AGENT_SECRET_KEY);
                    if (secretKey == null || secretKey.isEmpty()) {
                        throw new ServiceException(-1000, AGENT_SECRET_KEY + "Empty agent secret key");
                    }
                    logger.info("Get the agent secret key({})", secretKey);
                }
            }
        }
        return secretKey;
    }

    public static String getGateway() {
        if (gateway == null) {
            synchronized (DockerEnv.class) {
                if (gateway == null) {
                    gateway = getProperty(AGENT_GATEWAY);
                    if (gateway == null || gateway.isEmpty()) {
                        throw new ServiceException(-1000, AGENT_GATEWAY + "Empty agent gateway");
                    }
                    logger.info("Get the gateway({})", gateway);
                }
            }
        }
        return gateway;
    }

    public static String getJobPool() {
        if (jobPool == null) {
            synchronized (DockerEnv.class) {
                if (jobPool == null) {
                    jobPool = getProperty(JOB_POOL);
                    logger.info("Get the jobPool({})", jobPool);
                }
            }
        }
        return jobPool;
    }

    public static String getDockerHostIp() {
        if (dockerHostIp == null) {
            synchronized (DockerEnv.class) {
                if (dockerHostIp == null) {
                    dockerHostIp = getProperty(DOCKER_HOST_IP);
                    if (dockerHostIp == null || dockerHostIp.isEmpty()) {
                        throw new ServiceException(-1000, DOCKER_HOST_IP + "Empty dockerHostIp");
                    }
                    logger.info("Get the dockerHostIp({})", dockerHostIp);
                }
            }
        }
        return dockerHostIp;
    }

    public static String getDockerHostPort() {
        if (dockerHostPort == null) {
            synchronized (DockerEnv.class) {
                if (dockerHostPort == null) {
                    dockerHostPort = getProperty(DOCKER_HOST_PORT);
                    if (dockerHostPort == null || dockerHostPort.isEmpty()) {
                        throw new ServiceException(-1000, DOCKER_HOST_PORT + "Empty dockerHostPort");
                    }
                    logger.info("Get the dockerHostPort({})", dockerHostPort);
                }
            }
        }
        return dockerHostPort;
    }

    public static String getHostname() {
        if (hostname == null) {
            synchronized (DockerEnv.class) {
                if (hostname == null) {
                    hostname = getProperty(HOSTNAME);
                    if (hostname == null || hostname.isEmpty()) {
                        throw new ServiceException(-1000, HOSTNAME + "Empty hostname");
                    }
                    logger.info("Get the hostname({})", hostname);
                }
            }
        }
        return hostname;
    }

    private static String getProperty(String prop) {
        String value = System.getenv(prop);
        if (value == null || value.isEmpty()) {
            value = System.getProperty(prop);
        }
        return value;
    }
} 